<?php

namespace Candifly;

use Candifly\Mapper\ServiceMapper;
use Candifly\Exception\ForbiddenException;
use Candifly\Exception\BadRequestException;


if(!defined('ABSPATH'))
    exit();

define('MHASH', 'weOX16zqzrxCVME7Pfk8WtT5F6cRNrE0fAdHeXNuEWl5p7WJ');

class CandiRest
{
    /**
     * Initialize Rest API
     */
    public function init()
    {
        register_rest_route('candifly/v1/account', 'link', [
            'methods'   => 'POST',
            'callback'  => [$this, 'linkAccount']
        ]);
        
        register_rest_route('candifly/v1/account', 'create', [
            'methods'   => 'POST',
            'callback'  => [$this, 'createAccount']
        ]);
        
        register_rest_route('candifly/v1/account', 'delete', [
            'methods'   => 'POST',
            'callback'  => [$this, 'deleteAccount']
        ]);
        
        register_rest_route('candifly/v1/calendars', 'list', [
            'methods'   => 'POST',
            'callback'  => [$this, 'listCalendars']
        ]);
        
        register_rest_route('candifly/v1/app', 'updateAvailable', [
            'methods'   => 'POST',
            'callback'  => [$this, 'updateAvailable']
        ]);
    }
    
    /**
     * Deletes all account information 
     * 
     * @param   object $data
     * @return  \WP_REST_Response
     * @throws  ForbiddenException
     */
    public function rest_listCalendars($data)
    {
        $mapper = new ServiceMapper();
        $response = $mapper->setService('candifly')->getResponse('listCalendars');
        
        
        return new \WP_REST_Response([
                'success'   => true,
                'calendars' => $response->calendars
            ], 200);
    }
    
    /**
     * Checks if there is an update available
     * 
     * @return  \WP_REST_Response
     * @throws  ForbiddenException
     */
    public function rest_updateAvailable()
    {
        $verAvailable = \get_option(CANDIFLY_VERSION_AVAILABLE_OPTION);
        if(!$verAvailable || version_compare(Application::getConfig()->ver, $verAvailable) >= 0) {
            $mapper = new ServiceMapper();
            $response = $mapper->setService('candifly')->getResponse('checkVersion');
            \update_option(CANDIFLY_VERSION_AVAILABLE_OPTION, $response->version);
            $verAvailable = $response->version;
        }
        
        return new \WP_REST_Response([
                'success'           => true,
                'updateAvailable'   => (version_compare(Application::getConfig()->ver, $verAvailable) < 0) ? $verAvailable : null
            ], 200);
    }
    
    /**
     * Deletes all account information 
     * 
     * @param   object $data
     * @return  \WP_REST_Response
     * @throws  ForbiddenException
     */
    public function rest_deleteAccount($data)
    {
        delete_option(CANDIFLY_EMAIL_OPTION);
        delete_option(CANDIFLY_PASSWORD_OPTION);
        delete_option(CANDIFLY_SECRET_OPTION);
        delete_option(CANDIFLY_KEY_OPTION);
        
        return new \WP_REST_Response(['success' => true], 200);
    }
    
    /**
     * Link to an existing candifly account
     * 
     * @param   object $data
     * @return  \WP_REST_Response
     * @throws  ForbiddenException
     */
    public function rest_linkAccount($data)
    {
        $params = $data->get_params();
        if(!array_key_exists("username", $params) || !array_key_exists("password", $params))
            throw new ForbiddenException(910001, 'Missing credentials');

        $response = $this->getSecret($params);
        if(!$response->secret)
            return new \WP_REST_Response($response, 400);
        
        if(array_key_exists('save', $params)) {
            update_option(CANDIFLY_EMAIL_OPTION, $params['username']);
            update_option(CANDIFLY_PASSWORD_OPTION, $params['password']);
        }
        else {
            delete_option(CANDIFLY_EMAIL_OPTION);
            delete_option(CANDIFLY_PASSWORD_OPTION);
        }
        
        update_option(CANDIFLY_SECRET_OPTION, $response->secret);
        
        $token = self::getToken($response->secret);
        update_option(CANDIFLY_KEY_OPTION, $token);
        
        return new \WP_REST_Response([
                'success' => true,
                'session' => [
                    'email' => $params['username'],
                    'secret' => $response->secret,
                    'authKey' => $token
                ]
            ], 200);
    }
    
    /**
     * Creates an account and fetch secret
     * 
     * @param   stdClass $data
     * @return  \WP_REST_Response
     * @throws  ForbiddenException
     */
    public function rest_createAccount($data)
    {
        $params = $data->get_params();
        if(!array_key_exists("username", $params) || !array_key_exists("password", $params))
            throw new ForbiddenException(910002, 'Missing credentials');

        $params['hash'] = MHASH;
        $params['email'] = $params['username'];
        $mapper = new ServiceMapper();
        $response = $mapper->setService('candifly')->getResponse('register', $params);
        
        if(!$response->success)
            return new \WP_REST_Response($response, 400);
        
        $secretresponse = $this->getSecret($params);
        if(!$secretresponse->secret)
            return new \WP_REST_Response($response, 400);
        
        if(array_key_exists('save', $params)) {
            update_option(CANDIFLY_EMAIL_OPTION, $params['username']);
            update_option(CANDIFLY_PASSWORD_OPTION, $params['password']);
        }
        else {
            delete_option(CANDIFLY_EMAIL_OPTION);
            delete_option(CANDIFLY_PASSWORD_OPTION);
        }
        
        update_option(CANDIFLY_SECRET_OPTION, $secretresponse->secret);
        
        $token = self::getToken($secretresponse->secret);
        update_option(CANDIFLY_KEY_OPTION, $token);

        return new \WP_REST_Response([
            'success' => true,
            'session' => [
                'email' => $params['username'],
                'secret' => $secretresponse->secret,
                'authKey' => $token
                ]

            ], 200);
    }
    
    /**
     * Gets secret from user credentials
     * 
     * @param   array $params
     * @return  string
     */
    private function getSecret($params)
    {
        $mapper = new ServiceMapper();
        return $mapper->setService('candifly')->getResponse('secret', $params);
    }
    
    /**
     * Gets a new session token from secret key
     * 
     * @param   string $secret
     * @return  string
     * @throws  BadRequestException
     */
    public static function getToken($secret)
    {
        $mapper = new ServiceMapper();
        $response = $mapper->setService('candifly')->getResponse('createSession', ['secret' => $secret]);
        if(!$response->success)
            throw new BadRequestException(910003, 'Can`t get token - '.$response->error->description);
        
        return $response->authKey;
    }
    
    /**
     * Constructs an api JSON error message
     * 
     * @param   \Exception  $e
     * @param   int         $httpcode
     * @return  \WP_REST_Response
     */
    private function sendJsonError($e, $httpcode)
    {
        return new \WP_REST_Response([
            'success' => FALSE,
            'error' => [
                'code' => $e->getCode(), 
                'description' => $e->getMessage()
            ]
        ], $httpcode);
    }
    
    /**
     * Magic method to handle and call API functions
     * 
     * @param   string  $method
     * @param   array   $data
     * @return  \WP_REST_Response
     * @throws  BadRequestException
     * @throws  ForbiddenException
     */
    public function __call($method, $data)
    {
        try {
            if(!method_exists($this, 'rest_'.$method))
                throw new BadRequestException(910008, 'No such method - '.$method);

            $nonce = $data[0]->get_headers()['x_wp_nonce'][0];
            if(!wp_verify_nonce($nonce, 'wp_rest'))
                throw new ForbiddenException(910001, 'Invalid nonce');
            
            return call_user_func_array([$this, 'rest_'.$method], $data);
            
        } 
        catch (\Candifly\Exception\BadRequestException $e) {
            return $this->sendJsonError($e, 400);
        }
        catch (\Candifly\Exception\AuthenticateException $e) {
            return $this->sendJsonError($e, 401);
        }
        catch (\Candifly\Exception\ForbiddenException $e) {
            return $this->sendJsonError($e, 403);
        }
        catch (\Candifly\Exception\NotFoundException $e) {
            return $this->sendJsonError($e, 404);
        }
        catch (\Exception $e) {
            return $this->sendJsonError($e, 400);
        }
        
    }
    
}
